package com.chelvc.framework.base.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.math.RoundingMode;

import com.chelvc.framework.base.jackson.JacksonDecimalSerializer;
import com.fasterxml.jackson.annotation.JacksonAnnotationsInside;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * 小数格式化注解
 *
 * @author Woody
 * @date 2023/4/5
 * @see JacksonDecimalSerializer
 */
@Inherited
@Documented
@Target(ElementType.FIELD)
@Retention(RetentionPolicy.RUNTIME)
@JacksonAnnotationsInside
@JsonSerialize(using = JacksonDecimalSerializer.class)
public @interface Decimal {
    /**
     * 是否去掉多余的0
     *
     * @return true/false
     */
    boolean trimZeros() default false;

    /**
     * 获取小数保留位数，默认保留2位小数
     *
     * @return 小数保留位数
     */
    int scale() default JacksonDecimalSerializer.DEFAULT_SCALE;

    /**
     * 获取小数舍弃模式，默认四舍五入
     *
     * @return 小数舍弃模式
     */
    RoundingMode mode() default RoundingMode.HALF_UP;
}
