package com.chelvc.framework.base.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.chelvc.framework.base.model.Platform;
import com.chelvc.framework.base.model.Terminal;
import com.chelvc.framework.base.util.StringUtils;

/**
 * 版本号匹配注解
 *
 * @author Woody
 * @date 2023/4/5
 */
@Inherited
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.TYPE, ElementType.FIELD})
public @interface Version {
    /**
     * 获取匹配版本号
     *
     * @return 版本号
     */
    String value() default StringUtils.EMPTY;

    /**
     * 获取匹配版本号（当前版本号在此版本号之前）
     *
     * @return 版本号
     */
    String before() default StringUtils.EMPTY;

    /**
     * 获取匹配版本号（当前版本号在此版本号之后）
     *
     * @return 版本号
     */
    String after() default StringUtils.EMPTY;

    /**
     * 是否包含匹配版本号
     *
     * @return true/false
     */
    boolean contains() default false;

    /**
     * 获取匹配渠道数组
     *
     * @return 渠道数组
     */
    String[] channels() default {};

    /**
     * 获取匹配平台数组
     *
     * @return 平台数组
     */
    Platform[] platforms() default {};

    /**
     * 获取终端数组
     *
     * @return 终端数组
     */
    Terminal[] terminals() default {};
}
