package com.chelvc.framework.base.apidoc;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

import com.arsframework.apidoc.core.MethodAnalyser;
import com.arsframework.apidoc.core.Parameter;
import com.arsframework.apidoc.core.ParameterAnalyser;
import com.chelvc.framework.base.model.Enumerable;
import com.chelvc.framework.base.model.Paging;
import com.chelvc.framework.base.model.Period;
import com.chelvc.framework.base.model.Region;
import com.chelvc.framework.base.util.ObjectUtils;
import org.springframework.util.CollectionUtils;

/**
 * 方法解析器工厂实现
 *
 * @author Woody
 * @date 2023/4/5
 */
public class MethodAnalyserFactory implements MethodAnalyser.Factory {
    @Override
    public MethodAnalyser build(Method method) {
        return new MethodAnalyser(method) {
            @Override
            protected ParameterAnalyser getParameterAnalyser() {
                return new ParameterAnalyser(this.method) {
                    @Override
                    protected void afterInitializeParameter(Parameter parameter) {
                        if (parameter.getOriginal() == Paging.class) {
                            parameter.setType(Paging.class);
                            parameter.setFields(null);
                            parameter.setFormat("页码,分页大小");
                            parameter.setExample("1,10");
                        } else if (parameter.getOriginal() == Period.class) {
                            parameter.setType(Period.class);
                            parameter.setFields(null);
                            parameter.setFormat("开始时间,结束时间");
                            parameter.setExample("2023-01-01,2023-01-01 23:59:59");
                        } else if (parameter.getOriginal() == Region.class) {
                            parameter.setType(Integer.class);
                            parameter.setFields(null);
                            parameter.setExample("110000");
                        } else if (!parameter.isInput() && Enumerable.class.isAssignableFrom(parameter.getOriginal())) {
                            parameter.setType(Object.class);
                            List<Parameter.Option> options = parameter.getOptions();
                            Parameter.Option example = CollectionUtils.isEmpty(options) ? null : options.get(0);
                            List<Parameter> fields = Arrays.asList(
                                    Parameter.builder().input(false).type(String.class).original(String.class)
                                            .name("code").description("编码").options(options)
                                            .example(ObjectUtils.ifNull(example, Parameter.Option::getKey)).build(),
                                    Parameter.builder().input(false).type(String.class).original(String.class)
                                            .name("description").description("描述").options(options)
                                            .example(ObjectUtils.ifNull(example, Parameter.Option::getValue)).build()
                            );
                            parameter.setFields(fields);
                        }
                    }
                };
            }
        };
    }
}
