package com.chelvc.framework.base.config;

import com.chelvc.framework.base.jackson.JacksonNullSerializerModifier;
import com.chelvc.framework.base.jackson.JacksonSerializeModule;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ser.SerializerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;

/**
 * Jackson全局配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
public class JacksonConfigurer {
    @Bean
    @Primary
    public ObjectMapper objectMapper(Jackson2ObjectMapperBuilder builder) {
        ObjectMapper mapper = builder.build().setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY)
                .configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true)
                .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        SerializerFactory factory = mapper.getSerializerFactory();
        mapper.setSerializerFactory(factory.withSerializerModifier(new JacksonNullSerializerModifier()));
        mapper.registerModule(new JacksonSerializeModule());
        return mapper;
    }
}