package com.chelvc.framework.base.config;

import java.util.Collections;
import java.util.List;
import javax.annotation.PostConstruct;

import com.chelvc.framework.base.interceptor.StandardUnifiedResponseWrapper;
import com.chelvc.framework.base.interceptor.UnifiedResponseInterceptor;
import com.chelvc.framework.base.interceptor.UnifiedResponseWrapper;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 请求结果统一包装配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UnifiedResponseConfigurer {
    private final RequestMappingHandlerAdapter requestMappingHandlerAdapter;

    @Bean
    @ConditionalOnMissingBean(UnifiedResponseWrapper.class)
    public UnifiedResponseWrapper unifiedResponseWrapper() {
        return new StandardUnifiedResponseWrapper();
    }

    @Bean
    public UnifiedResponseInterceptor unifiedResponseInterceptor() {
        List<HandlerMethodReturnValueHandler> handlers = requestMappingHandlerAdapter.getReturnValueHandlers();
        RequestResponseBodyMethodProcessor processor = CollectionUtils.isEmpty(handlers) ? null :
                handlers.stream().filter(handler -> handler instanceof RequestResponseBodyMethodProcessor)
                        .map(handler -> (RequestResponseBodyMethodProcessor) handler).findAny().orElse(null);
        return new UnifiedResponseInterceptor(processor, this.unifiedResponseWrapper());
    }

    @PostConstruct
    public void register() {
        UnifiedResponseInterceptor unifiedResponseInterceptor = this.unifiedResponseInterceptor();
        List<HandlerMethodReturnValueHandler> handlers = this.requestMappingHandlerAdapter.getReturnValueHandlers();
        if (CollectionUtils.isEmpty(handlers)) {
            this.requestMappingHandlerAdapter.setReturnValueHandlers(Collections.singletonList(unifiedResponseInterceptor));
            return;
        }
        List<HandlerMethodReturnValueHandler> replaceHandlers = Lists.newArrayListWithCapacity(handlers.size() + 1);
        replaceHandlers.add(unifiedResponseInterceptor);
        replaceHandlers.addAll(handlers);
        this.requestMappingHandlerAdapter.setReturnValueHandlers(replaceHandlers);
    }
}
