package com.chelvc.framework.base.exception;

import com.chelvc.framework.base.util.ObjectUtils;
import lombok.Getter;
import lombok.NonNull;
import org.springframework.http.HttpStatus;

/**
 * 业务异常
 *
 * @author Woody
 * @date 2023/4/5
 */
@Getter
public class BusinessException extends PlatformException {
    /**
     * 异常数据
     */
    private final Object data;

    /**
     * 异常状态
     */
    private final HttpStatus status;

    public BusinessException() {
        this(HttpStatus.INTERNAL_SERVER_ERROR);
    }

    public BusinessException(String message) {
        this(HttpStatus.INTERNAL_SERVER_ERROR, message);
    }

    public BusinessException(HttpStatus status) {
        this(status, ObjectUtils.ifNull(status, HttpStatus::getReasonPhrase), null, null);
    }

    public BusinessException(HttpStatus status, String message) {
        this(status, message, null, null);
    }

    public BusinessException(HttpStatus status, String message, Object data) {
        this(status, message, data, null);
    }

    public BusinessException(HttpStatus status, Throwable cause) {
        this(status, ObjectUtils.ifNull(status, HttpStatus::getReasonPhrase), null, cause);
    }

    public BusinessException(@NonNull HttpStatus status, String message, Throwable cause) {
        this(status, message, null, cause);
    }

    public BusinessException(@NonNull HttpStatus status, String message, Object data, Throwable cause) {
        super(message, cause);
        this.status = status;
        this.data = data;
    }
}
