package com.chelvc.framework.base.model;

import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;

import lombok.Getter;
import lombok.ToString;

/**
 * 延时消息
 *
 * @param <T> 消息体类型
 * @author Woody
 * @date 2023/4/5
 */
@Getter
@ToString
public class DelayMessage<T> implements Delayed {
    /**
     * 消息体对象
     */
    private final T payload;

    /**
     * 过期时间戳
     */
    private final long expiration;

    public DelayMessage(T payload, long expiration) {
        this.payload = payload;
        this.expiration = expiration;
    }

    @Override
    public long getDelay(TimeUnit unit) {
        return unit.convert(this.expiration - System.currentTimeMillis(), TimeUnit.NANOSECONDS);
    }

    @Override
    public int compareTo(Delayed o) {
        return (int) (this.expiration - ((DelayMessage<?>) o).expiration);
    }
}
