package com.chelvc.framework.base.model;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

import com.google.common.collect.ImmutableMap;

/**
 * 可枚举对象接口
 *
 * @author Woody
 * @date 2023/4/5
 */
public interface Enumerable extends Serializable {
    /**
     * 获取枚举项名称
     *
     * @return 枚举项名称
     */
    default String name() {
        return null;
    }

    /**
     * 获取枚举编号
     *
     * @return 枚举编号
     */
    default String getCode() {
        return this.name();
    }

    /**
     * 获取排序数字
     *
     * @return 排序数字
     */
    default int getOrder() {
        return 0;
    }

    /**
     * 获取枚举描述
     *
     * @return 枚举描述
     */
    default String getDescription() {
        return this.getCode();
    }

    /**
     * 枚举对象结构化
     *
     * @return 枚举对象结构化信息
     */
    default Map<String, Object> structure() {
        String code = Objects.requireNonNull(this.getCode(), "Enum code must not be null");
        String description = Objects.requireNonNull(this.getDescription(), "Enum description must not be null");
        return ImmutableMap.of("code", code, "description", description);
    }
}
