package com.chelvc.framework.base.util;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;

import lombok.NonNull;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.codec.digest.MessageDigestAlgorithms;

/**
 * 编解码工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
public final class CodecUtils {
    private CodecUtils() {
    }

    /**
     * MD5加密
     *
     * @param original 明文
     * @return 密文
     */
    public static String md5(String original) {
        return encode(original, MessageDigestAlgorithms.MD5);
    }

    /**
     * SHA256加密
     *
     * @param original 明文
     * @return 密文
     */
    public static String sha256(String original) {
        return encode(original, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * SHA256加密
     *
     * @param original  明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static String encode(String original, @NonNull String algorithm) {
        if (original == null) {
            return null;
        }
        MessageDigest digest = DigestUtils.getDigest(algorithm);
        byte[] bytes = digest.digest(original.getBytes(StandardCharsets.UTF_8));
        return Hex.encodeHexString(bytes);
    }
}
