package com.chelvc.framework.base.util;

import java.nio.charset.StandardCharsets;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.safety.Whitelist;

/**
 * Jsoup工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
public final class JsoupUtils {
    /**
     * 白名单
     */
    private static final Whitelist WHITELIST = Whitelist.relaxed();

    /**
     * 文档输出设置
     */
    private static final Document.OutputSettings OUTPUT_SETTINGS =
            new Document.OutputSettings().charset(StandardCharsets.UTF_8).prettyPrint(false);

    static {
        // 初始化白名单
        WHITELIST.addTags("figure");
        WHITELIST.addAttributes(":all", "class");
        WHITELIST.addAttributes(":all", "style");
    }

    private JsoupUtils() {
    }

    /**
     * 清理数据
     *
     * @param text 文本内容
     * @return 文本内容
     */
    public static String clean(String text) {
        return StringUtils.isEmpty(text) ? text : Jsoup.clean(text, StringUtils.EMPTY, WHITELIST, OUTPUT_SETTINGS);
    }

    /**
     * 清理数据
     *
     * @param texts 文本内容数组
     * @return 文本内容数组
     */
    public static String[] clean(String[] texts) {
        if (texts == null || texts.length == 0) {
            return texts;
        }
        String[] cleans = new String[texts.length];
        for (int i = 0, size = texts.length; i < size; i++) {
            cleans[i] = clean(texts[i]);
        }
        return cleans;
    }

    /**
     * 过滤html标签
     *
     * @param text 文本内容
     * @return 文本内容
     */
    public static String text(String text) {
        return StringUtils.isEmpty(text) ? text : Jsoup.parse(text).text();
    }
}