package com.chelvc.framework.database.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.chelvc.framework.database.interceptor.PropertyUpdateInterceptor;

/**
 * 机密数据注解，使用该注解的字段值将会在入库前加密和出库后解密
 *
 * @author Woody
 * @date 2023/4/5
 * @see PropertyUpdateInterceptor
 */
@Inherited
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.TYPE, ElementType.FIELD})
public @interface Confidential {
    /**
     * 判断数据是否是机密数据
     *
     * @return true/false
     */
    boolean value() default true;
}
