package com.chelvc.framework.database.config;

import java.util.Collection;
import java.util.Comparator;

import com.baomidou.mybatisplus.core.incrementer.IdentifierGenerator;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.InnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.chelvc.framework.base.util.IdentityUtils;
import com.chelvc.framework.base.util.ObjectUtils;
import com.chelvc.framework.database.interceptor.DeletedExcludeHandler;
import com.chelvc.framework.database.interceptor.TenantIsolateHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;

/**
 * MybatisPlus配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MybatisPlusConfigurer {
    private final ApplicationContext applicationContext;

    @Bean
    public IdentifierGenerator identifierGenerator() {
        return entity -> IdentityUtils.generate();
    }

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();

        // 设置自定义租户隔离拦截器
        Collection<InnerInterceptor> wheres = this.applicationContext.getBeansOfType(InnerInterceptor.class).values();
        wheres.stream().sorted(Comparator.comparingInt(where ->
                ObjectUtils.ifNull(where.getClass().getAnnotation(Order.class), Order::value, () -> 0))
        ).forEach(interceptor::addInnerInterceptor);

        // 设置租户隔离拦截器
        interceptor.addInnerInterceptor(new TenantLineInnerInterceptor() {
            {
                this.setTenantLineHandler(new TenantIsolateHandler());
            }
        });

        // 设置删除数据过滤拦截器
        interceptor.addInnerInterceptor(new TenantLineInnerInterceptor() {
            {
                this.setTenantLineHandler(new DeletedExcludeHandler());
            }
        });

        // 设置分页拦截器，数据过滤条件与拦截器设置顺序有关，分页拦截器最后设置才能在分页时同时使用数据隔离过滤条件
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor());
        return interceptor;
    }
}