package com.chelvc.framework.database.entity;

import java.io.Serializable;

import com.chelvc.framework.database.interceptor.PropertyUpdateInterceptor;

/**
 * 可进行数据隔离的数据模型（多租户），租户信息自动初始化以及数据查询自动过滤
 *
 * @param <ID> 主键类型
 * @author Woody
 * @date 2023/4/5
 * @see BasicTenantEntity
 * @see BasicTenantTreeEntity
 * @see PropertyUpdateInterceptor
 */
public interface Isolatable<ID extends Serializable> extends Entity<ID> {
    /**
     * 获取租户编号
     *
     * @return 租户编号
     */
    Integer getTenant();

    /**
     * 设置租户编号
     *
     * @param tenant 租户编号
     */
    void setTenant(Integer tenant);
}
