package com.chelvc.framework.database.interceptor;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.fasterxml.jackson.core.type.TypeReference;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.springframework.util.CollectionUtils;

/**
 * 字典字段处理实现
 *
 * @author Woody
 * @date 2023/4/5
 */
public class MappingTypeHandler extends BaseTypeHandler<Map<?, ?>> {
    /**
     * 类型引用实例
     */
    private static final TypeReference<Map<?, ?>> REFERENCE = new TypeReference<Map<?, ?>>() {
    };

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, Map<?, ?> parameter, JdbcType jdbcType)
            throws SQLException {
        if (CollectionUtils.isEmpty(parameter)) {
            ps.setNull(i, Types.VARCHAR);
        } else {
            ps.setString(i, JacksonContextHolder.serialize(parameter));
        }
    }

    @Override
    public Map<?, ?> getNullableResult(ResultSet rs, String columnName) throws SQLException {
        return JacksonContextHolder.deserialize(rs.getString(columnName), REFERENCE);
    }

    @Override
    public Map<?, ?> getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        return JacksonContextHolder.deserialize(rs.getString(columnIndex), REFERENCE);
    }

    @Override
    public Map<?, ?> getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        return JacksonContextHolder.deserialize(cs.getString(columnIndex), REFERENCE);
    }
}
