package com.chelvc.framework.database.interceptor;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.database.config.MybatisPlusConfigurer;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.entity.Isolatable;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;

/**
 * 数据隔离拦截器，数据查询自动加入租户筛选逻辑
 *
 * @author Woody
 * @date 2023/4/5
 * @see MybatisPlusConfigurer
 */
public class TenantIsolateHandler implements TenantLineHandler {
    /**
     * 租户编号字段名称
     */
    public static final String TENANT_COLUMN_NAME = "tenant";

    @Override
    public Expression getTenantId() {
        return new LongValue(SessionContextHolder.getSession().getTenant());
    }

    @Override
    public String getTenantIdColumn() {
        return TENANT_COLUMN_NAME;
    }

    @Override
    public boolean ignoreTable(String tableName) {
        if (SessionContextHolder.getTenant(false) != null) {
            TableInfo table = DatabaseContextHolder.getTable(tableName);
            return table == null || !Isolatable.class.isAssignableFrom(table.getEntityType());
        }
        return true;
    }
}
