package com.chelvc.framework.dubbo.interceptor;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.common.constants.CommonConstants;
import org.apache.dubbo.common.extension.Activate;
import org.apache.dubbo.rpc.AsyncRpcResult;
import org.apache.dubbo.rpc.Filter;
import org.apache.dubbo.rpc.Invocation;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.Result;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.dubbo.rpc.RpcException;

/**
 * Dubbo 生产者处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Activate(group = CommonConstants.PROVIDER)
public class DubboProviderInterceptor implements Filter {
    @Override
    public Result invoke(Invoker<?> invoker, Invocation invocation) throws RpcException {
        // 初始化会话信息
        Session session = (Session) RpcContext.getServerAttachment()
                .getObjectAttachment(DubboConsumerInterceptor.SESSION_CONTEXT_NAME);
        if (log.isDebugEnabled()) {
            log.debug("[{}] [{}]: {}", invocation.getTargetServiceUniqueName(), invocation.getMethodName(), session);
        }
        SessionContextHolder.setSession(session);

        // 调用目标接口
        try {
            Result result = invoker.invoke(invocation);
            Object value = result.getValue();
            if (value == null) {
                return result;
            }

            // 对象包装
            DubboObjectAdapterWrapper<Object, ?> wrapper =
                    DubboObjectAdapterHolder.lookupObjectAdapter(value.getClass());
            if (wrapper == null) {
                return result;
            }
            value = wrapper.getAdapter().wrap(result.getValue());
            return AsyncRpcResult.newDefaultAsyncResult(value, invocation);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }
}
