package com.chelvc.framework.email.config;

import java.util.Properties;
import javax.mail.Authenticator;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 邮件配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
public class EmailConfigurer {
    @Bean
    public Session emailSession(EmailProperties properties) {
        Properties config = new Properties();
        config.setProperty("mail.host", properties.getHost());
        config.setProperty("mail.port", properties.getPort());
        config.setProperty("mail.store.protocol", properties.getProtocol());
        config.setProperty("mail.transport.protocol", properties.getProtocol());
        config.setProperty(String.format("mail.%s.auth", properties.getProtocol()), "true");
        config.setProperty(String.format("mail.%s.writetimeout", properties.getProtocol()), "10000");
        config.setProperty(String.format("mail.%s.connectiontimeout", properties.getProtocol()), "10000");
        if (Boolean.TRUE.equals(properties.getEncrypted())) {
            config.setProperty(String.format("mail.%s.starttls.enable", properties.getProtocol()), "true");
            config.setProperty(String.format("mail.%s.starttls.required", properties.getProtocol()), "true");
        }
        config.setProperty(String.format("mail.%s.socketFactory.port", properties.getProtocol()), properties.getPort());
        config.setProperty(String.format("mail.%s.socketFactory.class", properties.getProtocol()),
                "javax.net.ssl.SSLSocketFactory");
        return Session.getInstance(config, new Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(properties.getUsername(), properties.getPassword());
            }
        });
    }
}
