package com.chelvc.framework.feign.config;

import com.chelvc.framework.feign.interceptor.FeignExceptionInterceptor;
import com.chelvc.framework.feign.interceptor.FeignResponseInterceptor;
import com.fasterxml.jackson.databind.ObjectMapper;
import feign.codec.Decoder;
import feign.codec.Encoder;
import feign.codec.ErrorDecoder;
import feign.form.FormEncoder;
import feign.optionals.OptionalDecoder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.cloud.openfeign.FeignAutoConfiguration;
import org.springframework.cloud.openfeign.support.ResponseEntityDecoder;
import org.springframework.cloud.openfeign.support.SpringDecoder;
import org.springframework.cloud.openfeign.support.SpringEncoder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;

import static org.springframework.beans.factory.config.BeanDefinition.SCOPE_PROTOTYPE;

/**
 * Feign配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@AutoConfigureBefore(FeignAutoConfiguration.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FeignConfigurer {
    private final ObjectFactory<HttpMessageConverters> messageConverters;
    private final MappingJackson2HttpMessageConverter mappingJackson2HttpMessageConverter;

    @Bean
    public Decoder decoder() {
        ObjectMapper serializer = this.mappingJackson2HttpMessageConverter.getObjectMapper();
        Decoder delegate = new OptionalDecoder(new ResponseEntityDecoder(new SpringDecoder(this.messageConverters)));
        return new FeignResponseInterceptor(delegate, serializer);
    }

    @Bean
    @Primary
    @Scope(SCOPE_PROTOTYPE)
    public Encoder formEncoder() {
        return new FormEncoder(new SpringEncoder(this.messageConverters));
    }

    @Bean
    public ErrorDecoder errorDecoder() {
        return new FeignExceptionInterceptor();
    }
}
