package com.chelvc.framework.feign.interceptor;

import java.io.IOException;
import java.io.Reader;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.exception.BusinessException;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.feign.util.FeignUtils;
import feign.Response;
import feign.Util;
import feign.codec.ErrorDecoder;
import lombok.extern.slf4j.Slf4j;

/**
 * Feign调用异常处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class FeignExceptionInterceptor implements ErrorDecoder {
    @Override
    public Exception decode(String method, Response response) {
        // 获取异常信息
        String message;
        try (Reader reader = response.body().asReader(StandardCharsets.UTF_8)) {
            message = Util.toString(reader);
        } catch (IOException e) {
            return e;
        }

        // 如果是统一封装的消息则从消息体中获取异常信息
        Object data = null;
        if (FeignUtils.isUnifiedResponseBody(response)) {
            Result<?> result = Objects.requireNonNull(JacksonContextHolder.deserialize(message, Result.class));
            data = result.getData();
            message = result.getMessage();
        }
        return new BusinessException(HttpUtils.getStatus(response.status()), message, data);
    }
}
