package com.chelvc.framework.feign.interceptor;

import java.util.Enumeration;
import java.util.Map;
import java.util.Objects;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.base.util.ObjectUtils;
import com.google.common.collect.Maps;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * Feign请求头处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Component
public class FeignHeaderInterceptor implements RequestInterceptor {
    @Override
    public void apply(RequestTemplate template) {
        // 加载当前请求头
        Map<String, Object> headers = Maps.newHashMap();
        HttpServletRequest request = SessionContextHolder.getRequest();
        Enumeration<String> names = ObjectUtils.ifNull(request, HttpServletRequest::getHeaderNames);
        if (names != null) {
            while (names.hasMoreElements()) {
                String key = names.nextElement();
                headers.put(key, request.getHeader(key));
            }
        }

        // 重置会话请求头
        Session session = SessionContextHolder.getSession();
        headers.put(SessionContextHolder.HEADER_ID, session.getId());
        headers.put(SessionContextHolder.HEADER_MOBILE, session.getMobile());
        headers.put(SessionContextHolder.HEADER_DEVICE, session.getDevice());
        headers.put(SessionContextHolder.HEADER_TENANT, session.getTenant());
        headers.put(SessionContextHolder.HEADER_INITIAL, session.getInitial());
        headers.put(SessionContextHolder.HEADER_CHANNEL, session.getChannel());
        headers.put(SessionContextHolder.HEADER_PLATFORM, session.getPlatform());
        headers.put(SessionContextHolder.HEADER_TERMINAL, session.getTerminal());
        headers.put(SessionContextHolder.HEADER_VERSION, session.getVersion());
        headers.put(SessionContextHolder.HEADER_BUSINESS, session.getBusiness());
        headers.put(SessionContextHolder.HEADER_SEQUENCE, session.getSequence());
        headers.put(SessionContextHolder.HEADER_SIGNATURE, session.getSignature());

        // 初始化Feign请求头
        boolean debug = log.isDebugEnabled();
        headers.forEach((key, value) -> {
            if (Objects.nonNull(value)) {
                if (debug) {
                    log.debug("Initializing feign request header: {} = {}", key, value);
                }
                template.header(key, String.valueOf(value));
            }
        });
    }
}
