package com.chelvc.framework.feign.interceptor;

import java.io.IOException;
import java.io.Reader;
import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;

import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.feign.util.FeignUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import feign.FeignException;
import feign.Response;
import feign.codec.Decoder;
import lombok.NonNull;

/**
 * Feign响应处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
public class FeignResponseInterceptor implements Decoder {
    private final Decoder delegate;
    private final ObjectMapper serializer;

    public FeignResponseInterceptor(@NonNull Decoder delegate, @NonNull ObjectMapper serializer) {
        this.delegate = delegate;
        this.serializer = serializer;
    }

    @Override
    public Object decode(Response response, Type type) throws IOException, FeignException {
        if (FeignUtils.isUnifiedResponseBody(response)) {
            // 拆包统一封装的请求结果对象
            Result<?> result;
            try (Reader reader = response.body().asReader(StandardCharsets.UTF_8)) {
                result = this.serializer.readValue(reader, Result.class);
            }
            String message = this.serializer.writeValueAsString(result.getData());
            return this.delegate.decode(response.toBuilder().body(message, StandardCharsets.UTF_8).build(), type);
        }
        return this.delegate.decode(response, type);
    }
}
