package com.chelvc.framework.nacos.config;

import com.alibaba.cloud.nacos.NacosDiscoveryProperties;
import com.alibaba.cloud.nacos.registry.NacosRegistration;
import com.alibaba.cloud.nacos.registry.NacosServiceRegistry;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;

/**
 * 多服务Nacos配置
 * <p>
 * 自动查找工程中所有依赖服务的application.yml配置文件，并将其中的spring.application.name属性值作为服务名称自动注册到Nacos
 *
 * @author Woody
 * @date 2023/4/5
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MultiserverNacosConfigurer implements ApplicationRunner {
    private final ApplicationContext applicationContext;
    private final NacosServiceRegistry serviceRegistry;
    private final NacosDiscoveryProperties discoveryProperties;

    @Override
    public void run(ApplicationArguments args) {
        String applicationName = ApplicationContextHolder.getApplicationName(this.applicationContext);
        for (Resource resource : ApplicationContextHolder.getApplicationResources()) {
            // 获取应用服务名称
            String service = ApplicationContextHolder.getApplicationName(resource);
            if (StringUtils.isEmpty(service) || service.equals(applicationName)) {
                continue;
            }

            // 注册指定服务到Nacos
            this.discoveryProperties.setService(service);
            NacosRegistration registration = new NacosRegistration(null, this.discoveryProperties, applicationContext);
            this.serviceRegistry.register(registration);
        }
    }
}