package com.chelvc.framework.nacos.config;

import java.util.Properties;

import com.alibaba.boot.nacos.config.properties.NacosConfigProperties;
import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.PropertyKeyConst;
import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.exception.NacosException;
import com.alibaba.nacos.spring.context.annotation.config.NacosPropertySource;
import com.chelvc.framework.base.util.StringUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * Nacos配置中心配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@NacosPropertySource(dataId = "${nacos.config.id:${spring.application.name}}", autoRefreshed = true)
public class NacosConfigConfigurer {
    @Bean
    public ConfigService configService(NacosConfigProperties configuration) throws NacosException {
        Properties properties = new Properties();
        if (!StringUtils.isEmpty(configuration.getServerAddr())) {
            properties.put(PropertyKeyConst.SERVER_ADDR, configuration.getServerAddr());
        }
        if (!StringUtils.isEmpty(configuration.getContextPath())) {
            properties.put(PropertyKeyConst.CONTEXT_PATH, configuration.getContextPath());
        }
        if (!StringUtils.isEmpty(configuration.getEncode())) {
            properties.put(PropertyKeyConst.ENCODE, configuration.getEncode());
        }
        if (!StringUtils.isEmpty(configuration.getEndpoint())) {
            properties.put(PropertyKeyConst.ENDPOINT, configuration.getEndpoint());
        }
        if (!StringUtils.isEmpty(configuration.getNamespace())) {
            properties.put(PropertyKeyConst.NAMESPACE, configuration.getNamespace());
        }
        if (!StringUtils.isEmpty(configuration.getAccessKey())) {
            properties.put(PropertyKeyConst.ACCESS_KEY, configuration.getAccessKey());
        }
        if (!StringUtils.isEmpty(configuration.getSecretKey())) {
            properties.put(PropertyKeyConst.SECRET_KEY, configuration.getSecretKey());
        }
        return NacosFactory.createConfigService(properties);
    }
}
