package com.chelvc.framework.redis.config;

import java.util.Objects;

import com.chelvc.framework.base.util.StringUtils;
import lombok.NonNull;
import org.springframework.data.redis.serializer.StringRedisSerializer;

/**
 * 带前缀的Redis序列化处理器实现
 *
 * @author Woody
 * @date 2023/4/5
 */
public class PrefixRedisSerializer extends StringRedisSerializer {
    /**
     * Key前缀
     */
    protected final String prefix;

    public PrefixRedisSerializer(@NonNull String prefix) {
        this.prefix = prefix;
    }

    @Override
    public byte[] serialize(String string) {
        if (Objects.nonNull(string) && StringUtils.nonEmpty(this.prefix)) {
            string = this.prefix + string;
        }
        return super.serialize(string);
    }

    @Override
    public String deserialize(byte[] bytes) {
        String key = super.deserialize(bytes);
        return StringUtils.isEmpty(key) || StringUtils.isEmpty(this.prefix) ? key : key.substring(this.prefix.length());
    }
}
