package com.chelvc.framework.rocketmq.interceptor;

import java.util.List;

import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import com.chelvc.framework.rocketmq.model.ConsumeStatus;
import com.chelvc.framework.rocketmq.model.MessageContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeOrderlyContext;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.util.CollectionUtils;

/**
 * 会话消息监听器接口
 *
 * @author Woody
 * @date 2023/5/18
 */
public interface SessionRocketMQListener<T> extends RocketMQListener<T> {
    /**
     * 获取消息批量消费数量
     *
     * @return 消息数量
     */
    default int getBatchConsumeSize() {
        return 1;
    }

    /**
     * 获取顺序消费消息重试时间间隔
     *
     * @return 时间间隔
     * @see ConsumeOrderlyContext
     */
    default long getOrderlyRetryInterval() {
        return -1;
    }

    /**
     * 获取并发消费消息重试时间间隔
     *
     * @return 时间间隔
     * @see ConsumeConcurrentlyContext
     */
    default int getConcurrentlyRetryInterval() {
        return 0;
    }

    /**
     * 批量处理消息上下文信息
     *
     * @param contexts 消息上下文对象列表
     * @return 消费状态
     */
    default ConsumeStatus onMessageContexts(List<MessageContext<T>> contexts) {
        if (!CollectionUtils.isEmpty(contexts)) {
            contexts.forEach(context -> RocketMQContextHolder.consumeMessageContext(context, this::onMessage));
        }
        return ConsumeStatus.SUCCESS;
    }
}
