package com.chelvc.framework.security.config;

import java.util.List;

import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.Data;
import lombok.Getter;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 安全配置属性
 *
 * @author Woody
 * @date 2023/4/5
 * @see WebSecurityConfigurer
 * @see SecurityContextHolder
 */
@Getter
@Configuration
@ConfigurationProperties("platform.security")
public class SecurityProperties {
    /**
     * 令牌安全配置
     */
    private final Token token = new Token();

    /**
     * 资源安全配置
     */
    private final Resource resource = new Resource();

    /**
     * 数字签名配置属性
     */
    private final Signature signature = new Signature();

    /**
     * 资源安全配置
     */
    @Data
    public static class Resource {
        /**
         * 登录接口地址
         */
        private String login;

        /**
         * 注销接口地址
         */
        private String logout;

        /**
         * 拒绝的资源地址，多个地址使用","号隔开
         */
        private String deny;

        /**
         * 允许的资源地址，多个地址使用","号隔开
         */
        private String permit;

        /**
         * 忽略的资源地址，多个地址使用","号隔开
         */
        private String ignore;

        /**
         * 需要认证的资源地址，多个地址使用","号隔开
         */
        private String authenticated;

        /**
         * 资源访问控制列表
         */
        private List<Access> accesses;
    }

    /**
     * 资源访问控制
     */
    @Data
    public static class Access {
        /**
         * 资源地址，多个地址使用","号隔开
         */
        private String uri;

        /**
         * 客户端IP
         */
        private String ip;

        /**
         * 客户端角色
         */
        private String role;

        /**
         * 客户端权限
         */
        private String authority;

        /**
         * 访问许可
         */
        private String permission;
    }

    /**
     * 数字签名配置
     */
    @Data
    public static class Signature {
        /**
         * 是否启用
         */
        private Boolean enable = true;

        /**
         * 签名密钥
         */
        private String secret = "signature@secret";

        /**
         * 签名过期时间（秒）
         */
        private Integer expiration = 3 * 60;
    }

    /**
     * 令牌配置
     */
    @Data
    public static class Token {
        /**
         * 令牌密钥对配置实例
         */
        private final KeyPair keyPair = new KeyPair();
        /**
         * 令牌签名密钥
         */
        private String secret;
    }

    /**
     * 令牌密钥对配置
     */
    @Data
    public static class KeyPair {
        /**
         * 密钥别名
         */
        private String alias;

        /**
         * 资源路径
         */
        private String resource;

        /**
         * 密钥对密码
         */
        private String keyPairPassword;

        /**
         * 密钥存储密码
         */
        private String keyStorePassword;
    }
}
