package com.chelvc.framework.security.config;

import com.chelvc.framework.security.token.RedisTokenActiveValidator;
import com.chelvc.framework.security.token.RedisTokenReplayValidator;
import com.chelvc.framework.security.token.TokenActiveValidator;
import com.chelvc.framework.security.token.TokenReplayValidator;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * 会话安全相关配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@ConditionalOnClass(RedisTemplate.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SessionSecurityConfigurer {
    private final RedisTemplate<String, Object> redisTemplate;

    @Bean
    @ConditionalOnMissingBean(TokenActiveValidator.class)
    public TokenActiveValidator tokenActiveValidator() {
        return new RedisTokenActiveValidator(this.redisTemplate);
    }

    @Bean
    @ConditionalOnMissingBean(TokenReplayValidator.class)
    public TokenReplayValidator tokenReplayValidator() {
        return new RedisTokenReplayValidator(this.redisTemplate);
    }
}
