package com.chelvc.framework.security.context;

import java.util.Objects;

import com.chelvc.framework.base.util.StringUtils;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

/**
 * @author Woody
 * @date 2023/4/5
 */
@Component
public class PasswordContextHolder implements ApplicationContextAware {
    /**
     * 密码加解密处理器
     */
    private static PasswordEncoder PASSWORD_ENCODER = null;

    /**
     * 获取密码加解密处理器
     *
     * @return 密码加解密处理器
     */
    public static PasswordEncoder getPasswordEncoder() {
        return Objects.requireNonNull(PASSWORD_ENCODER, "Password encoder has not been initialized");
    }

    /**
     * 编码密码明文
     *
     * @param plaintext 密码文明
     * @return 密码密文
     */
    public static String encode(CharSequence plaintext) {
        return getPasswordEncoder().encode(plaintext);
    }

    /**
     * 判断密码是否匹配
     *
     * @param plaintext  密码文明
     * @param ciphertext 密码密文
     * @return true/false
     */
    public static boolean matches(CharSequence plaintext, String ciphertext) {
        return !StringUtils.isEmpty(plaintext) && !StringUtils.isEmpty(ciphertext)
                && getPasswordEncoder().matches(plaintext, ciphertext);
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        PASSWORD_ENCODER = applicationContext.getBean(PasswordEncoder.class);
    }
}
