package com.chelvc.framework.security.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.security.annotation.Permissible;
import com.chelvc.framework.security.context.SecurityContextHolder;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 接口权限验证拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Component
public class PermissionValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        // 判断接口是否需要认证
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod method = (HandlerMethod) handler;
        if (!method.getMethod().isAnnotationPresent(Permissible.class)
                && !method.getBeanType().isAnnotationPresent(Permissible.class)) {
            return true;
        }

        // 认证接口权限
        if (!SecurityContextHolder.hasPermission(request.getMethod(), request.getRequestURI())) {
            throw new AccessDeniedException("Access denied");
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).order(Integer.MIN_VALUE + 1).addPathPatterns("/**");
    }
}
