package com.chelvc.framework.security.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.HttpUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 安全异常处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@RestControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class SecurityExceptionInterceptor {
    /**
     * 认证异常转换
     *
     * @param response  请求响应对象
     * @param exception 异常对象
     * @return 请求结果
     */
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    @ExceptionHandler(BadCredentialsException.class)
    public Result<?> exception(HttpServletRequest request, HttpServletResponse response,
                               BadCredentialsException exception) {
        log.warn(HttpUtils.getExceptionMessage(request, HttpStatus.UNAUTHORIZED, exception));
        HttpUtils.initializeUnifiedResponseHeader(response);
        return Result.build(HttpStatus.UNAUTHORIZED);
    }

    /**
     * 拒绝访问异常转换
     *
     * @param response  请求响应对象
     * @param exception 异常对象
     * @return 请求结果
     */
    @ResponseStatus(HttpStatus.FORBIDDEN)
    @ExceptionHandler(AccessDeniedException.class)
    public Result<?> exception(HttpServletRequest request, HttpServletResponse response,
                               AccessDeniedException exception) {
        log.warn(HttpUtils.getExceptionMessage(request, HttpStatus.FORBIDDEN, exception));
        HttpUtils.initializeUnifiedResponseHeader(response);
        return Result.build(HttpStatus.FORBIDDEN);
    }
}
