package com.chelvc.framework.security.token;

import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

/**
 * 基于Redis的令牌存储实现
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class RedisTokenStore extends JwtTokenStore {
    private final RedisTemplate<String, Object> redisTemplate;

    public RedisTokenStore(@NonNull RedisTemplate<String, Object> redisTemplate,
                           @NonNull JwtAccessTokenConverter jwtAccessTokenConverter) {
        super(jwtAccessTokenConverter);
        this.redisTemplate = redisTemplate;
    }

    @Override
    public void storeAccessToken(OAuth2AccessToken token, OAuth2Authentication authentication) {
        SecurityContextHolder.storeAccessToken(token, this.redisTemplate);
    }

    @Override
    public void removeAccessToken(OAuth2AccessToken token) {
        SecurityContextHolder.removeAccessToken(token, this.redisTemplate);
    }
}
