package com.chelvc.framework.sms;

import java.util.function.Consumer;

/**
 * 短信验证码操作接口
 *
 * @author Woody
 * @date 2023/4/5
 */
public interface CaptchaSmsHandler {
    /**
     * 发送验证码
     *
     * @param mobile 手机号
     * @return 短信会话信息
     */
    SmsSession send(String mobile);

    /**
     * 获取验证码信息
     *
     * @param mobile 手机号
     * @return 验证码信息
     */
    Captcha getCaptcha(String mobile);

    /**
     * 使用验证码（使用后移除）
     *
     * @param mobile   手机号
     * @param consumer 验证码信息回调方法
     */
    void using(String mobile, Consumer<Captcha> consumer);

    /**
     * 检测验证码是否正确
     *
     * @param token  令牌标识
     * @param mobile 手机号
     * @param code   验证码
     * @return true/false
     */
    boolean check(String token, String mobile, String code);

    /**
     * 校验验证码是否正确（校验成功后移除）
     *
     * @param token  令牌标识
     * @param mobile 手机号
     * @param code   验证码
     * @return true/false
     */
    boolean validate(String token, String mobile, String code);
}
