package com.chelvc.framework.sms.config;

import com.aliyun.dysmsapi20170525.Client;
import com.aliyun.teaopenapi.models.Config;
import com.github.qcloudsms.SmsSingleSender;
import com.github.qcloudsms.httpclient.PoolingHTTPClient;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 短信配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SmsConfigurer {
    /**
     * 构建阿里云短信配置实例
     *
     * @param properties 配置属性
     * @return 配置实例
     */
    @Bean
    @ConditionalOnBean(AliyunSmsProperties.class)
    public Config aliyunSmsConfig(AliyunSmsProperties properties) {
        return new Config().setAccessKeyId(properties.getId())
                .setAccessKeySecret(properties.getSecret()).setEndpoint("dysmsapi.aliyuncs.com");
    }

    /**
     * 构建阿里云短信客户端实例
     *
     * @param config 配置信息
     * @return 客户端实例
     */
    @Bean
    @ConditionalOnBean(Config.class)
    public Client aliyunSmsClient(Config config) {
        try {
            return new Client(config);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 构建腾讯云短信发送处理器实例
     *
     * @param properties 配置属性
     * @return 短信发送处理器实例
     */
    @Bean
    @ConditionalOnBean(TencentSmsProperties.class)
    public SmsSingleSender tencentSmsSender(TencentSmsProperties properties) {
        return new SmsSingleSender(properties.getId(), properties.getSecret(), new PoolingHTTPClient());
    }
}
