package com.chelvc.framework.upload.config;

import com.chelvc.framework.base.config.HttpClientProperties;
import com.chelvc.framework.base.util.StringUtils;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.region.Region;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 文件上传配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UploadConfigurer {
    private final HttpClientProperties httpClientProperties;

    @Bean
    @ConditionalOnBean(TencentUploadProperties.class)
    public ClientConfig tencentUploadConfig(TencentUploadProperties properties) {
        ClientConfig config = new ClientConfig();
        config.setHttpProtocol(HttpProtocol.https);
        if (!StringUtils.isEmpty(properties.getRegion())) {
            config.setRegion(new Region(properties.getRegion()));
        }
        config.setMaxConnectionsCount(this.httpClientProperties.getMaxTotal());
        config.setConnectionTimeout(this.httpClientProperties.getConnectTimeout());
        config.setSocketTimeout(this.httpClientProperties.getSocketTimeout());
        config.setConnectionRequestTimeout(this.httpClientProperties.getConnectionRequestTimeout());
        return config;
    }

    @Bean(destroyMethod = "shutdown")
    @ConditionalOnBean(TencentUploadProperties.class)
    public COSClient tencentUploadClient(ClientConfig config, TencentUploadProperties properties) {
        COSCredentials credentials = new BasicCOSCredentials(properties.getAppid(), properties.getSecret());
        return new COSClient(credentials, config);
    }
}
