package com.chelvc.framework.wechat;

import java.util.Map;
import java.util.function.Supplier;

/**
 * 微信操作接口
 *
 * @author Woody
 * @date 2023/4/5
 */
public interface WechatHandler {
    /**
     * 获取AppID
     *
     * @return AppID
     */
    String getAppid();

    /**
     * 获取微信访问令牌
     *
     * @return 访问令牌
     */
    String getAccessToken();

    /**
     * 刷新微信访问令牌
     *
     * @return 访问令牌
     */
    String refreshAccessToken();

    /**
     * 获取微信访问令牌
     *
     * @param appid  AppID
     * @param secret AppSecret
     * @return 访问令牌
     */
    String getAccessToken(String appid, String secret);

    /**
     * 刷新微信访问令牌
     *
     * @param appid  AppID
     * @param secret AppSecret
     * @return 访问令牌
     */
    String refreshAccessToken(String appid, String secret);

    /**
     * 获取小程序Scheme码
     *
     * @return 小程序Scheme码
     */
    String getScheme();

    /**
     * 获取小程序Scheme码
     *
     * @param path        进入的小程序页面路径
     * @param query       进入小程序时的 query
     * @param environment 环境标识（正式版为"release"，体验版为"trial"，开发版为"develop"）
     * @return 小程序Scheme码
     */
    String getScheme(String path, String query, String environment);

    /**
     * 获取微信用户信息
     *
     * @param accessToken 访问令牌
     * @param openid      openid
     * @return 微信用户信息
     */
    WechatUser getUser(String accessToken, String openid);

    /**
     * 获取微信用户信息
     *
     * @param accessToken    访问令牌
     * @param openid         openid
     * @param tokenRefresher 令牌刷新方法
     * @return 微信用户信息
     */
    WechatUser getUser(String accessToken, String openid, Supplier<String> tokenRefresher);

    /**
     * 根据临时令牌获取手机号
     *
     * @param code 临时令牌
     * @return 手机号
     */
    String code2mobile(String code);

    /**
     * 根据临时令牌获取会话信息
     *
     * @param code 临时令牌
     * @return 会话信息
     */
    WechatSession code2session(String code);

    /**
     * 参数签名
     *
     * @param type       支付类型
     * @param parameters 签名参数
     * @return 签名信息
     */
    String sign(String type, Map<String, String> parameters);

    /**
     * 微信统一下单
     *
     * @param type    支付类型
     * @param request 支付请求参数
     * @return 微信统一下单信息
     */
    WechatUnifiedOrder unifiedorder(String type, WechatPayRequest request);
}
