package com.chelvc.framework.wechat.config;

import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.wechat.DefaultWechatPublicHandler;
import com.chelvc.framework.wechat.WechatHandler;
import com.chelvc.framework.wechat.WechatPublicHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

/**
 * 微信配置类
 *
 * @author Woody
 * @date 2023/4/5
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class WechatConfigurer implements BeanPostProcessor {
    private final RestTemplate restTemplate;
    private final WechatHandler wechatHandler;
    private final WechatProperties properties;
    private final GenericApplicationContext applicationContext;
    private boolean initialized;

    /**
     * 初始化微信公众号处理器
     */
    private void initializeWechatPublicHandler() {
        if (!this.initialized && !CollectionUtils.isEmpty(this.properties.getPublics())) {
            this.properties.getPublics().forEach(config -> {
                WechatPublicHandler handler = new DefaultWechatPublicHandler(
                        this.restTemplate, this.wechatHandler, config
                );
                String name = StringUtils.ifEmpty(config.getId(), config.getAppid());
                RootBeanDefinition definition = new RootBeanDefinition(WechatPublicHandler.class, () -> handler);
                this.applicationContext.registerBeanDefinition(name, definition);
            });
            this.initialized = true;
        }
    }

    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
        this.initializeWechatPublicHandler();
        return bean;
    }
}
