package com.chelvc.framework.wechat.config;

import java.util.Collections;
import java.util.List;

import com.chelvc.framework.wechat.PayMode;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 微信配置
 *
 * @author Woody
 * @date 2023/4/5
 */
@Data
@Configuration
@ConfigurationProperties("platform.wechat")
public class WechatProperties {
    /**
     * AppID
     */
    private String appid;

    /**
     * AppSecret
     */
    private String secret;

    /**
     * 缓存配置
     */
    private Cache cache = new Cache();

    /**
     * 公众号配置列表
     */
    private List<Public> publics = Collections.emptyList();

    /**
     * 支付配置列表
     */
    private List<Payment> payments = Collections.emptyList();

    /**
     * 公众号配置
     */
    @Data
    public static class Public {
        /**
         * 配置ID
         */
        private String id;

        /**
         * 数据密钥
         */
        private String key;

        /**
         * AppID
         */
        private String appid;

        /**
         * 公众号令牌
         */
        private String token;

        /**
         * AppSecret
         */
        private String secret;
    }

    /**
     * 支付配置
     */
    @Data
    public static class Payment {
        /**
         * 支付类型
         */
        private String type;

        /**
         * 支付方式
         */
        private PayMode mode;

        /**
         * AppID
         */
        private String appid;

        /**
         * 商户标识
         */
        private String mchid;

        /**
         * 商户密钥
         */
        private String mchkey;

        /**
         * 微信回调地址
         */
        private String callback;
    }

    /**
     * Reids缓存配置
     */
    @Data
    public static class Cache {
        /**
         * 主机地址
         */
        private String host;

        /**
         * 主机端口
         */
        private Integer port;

        /**
         * 登陆密码
         */
        private String password;

        /**
         * 数据库标识
         */
        private Integer database;
    }
}
